@@ -8,10 +8,10 @@ from importlib import import_module
 
 from saml2.s_utils import factory
 from saml2.s_utils import do_ava
-from saml2 import saml
+from saml2 import saml, ExtensionElement, NAMESPACE
 from saml2 import extension_elements_to_elements
 from saml2 import SAMLError
-from saml2.saml import NAME_FORMAT_UNSPECIFIED
+from saml2.saml import NAME_FORMAT_UNSPECIFIED, NAMEID_FORMAT_PERSISTENT, NameID
 
 import logging
 logger = logging.getLogger(__name__)
@@ -40,11 +40,8 @@ def load_maps(dirspec):
     for fil in os.listdir(dirspec):
         if fil.endswith(".py"):
             mod = import_module(fil[:-3])
-            for key, item in mod.__dict__.items():
-                if key.startswith("__"):
-                    continue
-                if isinstance(item, dict) and "to" in item and "fro" in item:
-                    mapd[item["identifier"]] = item
+            for item in _find_maps_in_module(mod):
+                mapd[item["identifier"]] = item
 
     return mapd
 
@@ -54,7 +51,7 @@ def ac_factory(path=""):
 
     :param path: The path to a directory where the attribute maps are expected
         to reside.
-    :return: A AttributeConverter instance
+    :return: A list of AttributeConverter instances
     """
     acs = []
 
@@ -62,51 +59,49 @@ def ac_factory(path=""):
         if path not in sys.path:
             sys.path.insert(0, path)
 
-        for fil in os.listdir(path):
+        for fil in sorted(os.listdir(path)):
             if fil.endswith(".py"):
                 mod = import_module(fil[:-3])
-                for key, item in mod.__dict__.items():
-                    if key.startswith("__"):
-                        continue
-                    if isinstance(item,
-                                  dict) and "to" in item and "fro" in item:
-                        atco = AttributeConverter(item["identifier"])
-                        atco.from_dict(item)
-                        acs.append(atco)
+                acs.extend(_attribute_map_module_to_acs(mod))
     else:
         from saml2 import attributemaps
 
         for typ in attributemaps.__all__:
             mod = import_module(".%s" % typ, "saml2.attributemaps")
-            for key, item in mod.__dict__.items():
-                if key.startswith("__"):
-                    continue
-                if isinstance(item, dict) and "to" in item and "fro" in item:
-                    atco = AttributeConverter(item["identifier"])
-                    atco.from_dict(item)
-                    acs.append(atco)
+            acs.extend(_attribute_map_module_to_acs(mod))
 
     return acs
 
 
-def ac_factory_II(path):
-    return ac_factory(path)
+def _attribute_map_module_to_acs(module):
+    """Scan an attribute map module and return any attribute maps defined
+
+    :param: module: the python map module
+    :type: types.ModuleType
+    :return: a generator yielding AttributeConverter defintions
+    :rtype: typing.Iterable[AttributeConverter]
+    """
+    for item in _find_maps_in_module(module):
+        atco = AttributeConverter(item["identifier"])
+        atco.from_dict(item)
+        yield atco
 
 
-# def ava_fro(acs, statement):
-#     """  Translates attributes according to their name_formats into the local
-#      names.
-#
-#     :param acs: AttributeConverter instances
-#     :param statement: A SAML statement
-#     :return: A dictionary with attribute names replaced with local names.
-#     """
-#     if not statement:
-#         return {}
-#
-#     acsdic = dict([(ac.name_format, ac) for ac in acs])
-#     acsdic[None] = acsdic[NAME_FORMAT_URI]
-#     return dict([acsdic[a.name_format].ava_from(a) for a in statement])
+def _find_maps_in_module(module):
+    """Find attribute map dictionaries in a map file
+
+    :param: module: the python map module
+    :type: types.ModuleType
+    :return: a generator yielding dict objects which have the right shape
+    :rtype: typing.Iterable[dict]
+    """
+    for key, item in module.__dict__.items():
+        if key.startswith("__"):
+            continue
+        if isinstance(item, dict) and "identifier" in item and (
+            "to" in item or "fro" in item
+        ):
+            yield item
 
 
 def to_local(acs, statement, allow_unknown_attributes=False):
@@ -118,42 +113,7 @@ def to_local(acs, statement, allow_unknown_attributes=False):
     :param allow_unknown_attributes: If unknown attributes are allowed
     :return: A key,values dictionary
     """
-    if not acs:
-        acs = [AttributeConverter()]
-        acsd = {"": acs}
-    else:
-        acsd = dict([(a.name_format, a) for a in acs])
-
-    ava = {}
-    for attr in statement.attribute:
-        try:
-            _func = acsd[attr.name_format].ava_from
-        except KeyError:
-            if attr.name_format == NAME_FORMAT_UNSPECIFIED or \
-                    allow_unknown_attributes:
-                _func = acs[0].lcd_ava_from
-            else:
-                logger.info("Unsupported attribute name format: %s",
-                    attr.name_format)
-                continue
-
-        try:
-            key, val = _func(attr)
-        except KeyError:
-            if allow_unknown_attributes:
-                key, val = acs[0].lcd_ava_from(attr)
-            else:
-                logger.info("Unknown attribute name: %s", attr)
-                continue
-        except AttributeError:
-            continue
-
-        try:
-            ava[key].extend(val)
-        except KeyError:
-            ava[key] = val
-
-    return ava
+    return list_to_local(acs, statement.attribute, allow_unknown_attributes)
 
 
 def list_to_local(acs, attrlist, allow_unknown_attributes=False):
@@ -246,7 +206,7 @@ def get_local_name(acs, attr, name_format):
     for aconv in acs:
         #print(ac.format, name_format)
         if aconv.name_format == name_format:
-            return aconv._fro[attr]
+            return aconv._fro.get(attr)
 
 
 def d_to_local_name(acs, attr):
@@ -313,23 +273,15 @@ class AttributeConverter(object):
 
     def lcd_ava_from(self, attribute):
         """
-        In nothing else works, this should
+        If nothing else works, this should
 
-        :param attribute: An Attribute Instance
+        :param attribute: an Attribute instance
         :return:
         """
-        try:
-            name = attribute.friendly_name.strip()
-        except AttributeError:
-            name = attribute.name.strip()
-
-        values = []
-        for value in attribute.attribute_value:
-            if not value.text:
-                values.append('')
-            else:
-                values.append(value.text.strip())
-
+        name = attribute.name.strip()
+        values = [
+            (value.text or '').strip()
+            for value in attribute.attribute_value]
         return name, values
 
     def fail_safe_fro(self, statement):
@@ -377,14 +329,17 @@ class AttributeConverter(object):
                 ext = extension_elements_to_elements(value.extension_elements,
                                                      [saml])
                 for ex in ext:
-                    cval = {}
-                    for key, (name, typ, mul) in ex.c_attributes.items():
-                        exv = getattr(ex, name)
-                        if exv:
-                            cval[name] = exv
-                    if ex.text:
-                        cval["value"] = ex.text.strip()
-                    val.append({ex.c_tag: cval})
+                    if attr == "eduPersonTargetedID" and ex.text:
+                        val.append(ex.text.strip())
+                    else:
+                        cval = {}
+                        for key, (name, typ, mul) in ex.c_attributes.items():
+                            exv = getattr(ex, name)
+                            if exv:
+                                cval[name] = exv
+                        if ex.text:
+                            cval["value"] = ex.text.strip()
+                        val.append({ex.c_tag: cval})
             elif not value.text:
                 val.append('')
             else:
@@ -488,20 +443,65 @@ class AttributeConverter(object):
         """
         attributes = []
         for key, value in attrvals.items():
-            lkey = key.lower()
-            try:
+            name = self._to.get(key.lower())
+            if name:
+                if name == "urn:oid:1.3.6.1.4.1.5923.1.1.1.10":
+                    # special case for eduPersonTargetedID
+                    attr_value = self.to_eptid_value(value)
+                else:
+                    attr_value = do_ava(value)
                 attributes.append(factory(saml.Attribute,
-                                          name=self._to[lkey],
+                                          name=name,
                                           name_format=self.name_format,
                                           friendly_name=key,
-                                          attribute_value=do_ava(value)))
-            except KeyError:
+                                          attribute_value=attr_value))
+            else:
                 attributes.append(factory(saml.Attribute,
                                           name=key,
                                           attribute_value=do_ava(value)))
 
         return attributes
 
+    def to_eptid_value(self, values):
+        """
+        Create AttributeValue instances of NameID from the given values.
+
+        Special handling for the "eptid" attribute
+        Name=urn:oid:1.3.6.1.4.1.5923.1.1.1.10
+        FriendlyName=eduPersonTargetedID
+
+        values is a list of items of type str or dict. When an item is a
+        dictionary it has the keys: "NameQualifier", "SPNameQualifier", and
+        "text".
+
+        Returns a list of AttributeValue instances of NameID elements.
+        """
+
+        if type(values) is not list:
+            values = [values]
+
+        def _create_nameid_ext_el(value):
+            text = value["text"] if isinstance(value, dict) else value
+            attributes = (
+                {
+                    "Format": NAMEID_FORMAT_PERSISTENT,
+                    "NameQualifier": value["NameQualifier"],
+                    "SPNameQualifier": value["SPNameQualifier"],
+                }
+                if isinstance(value, dict)
+                else {"Format": NAMEID_FORMAT_PERSISTENT}
+            )
+            element = ExtensionElement(
+                "NameID", NAMESPACE, attributes=attributes, text=text
+            )
+            return element
+
+        attribute_values = [
+            saml.AttributeValue(extension_elements=[_create_nameid_ext_el(v)])
+            for v in values
+        ]
+        return attribute_values
+
 
 class AttributeConverterNOOP(AttributeConverter):
     """ Does a NOOP conversion, that is no conversion is made """
