@@ -9,22 +9,25 @@ import threading
 import six
 import time
 import logging
+from warnings import warn as _warn
 
 from saml2.entity import Entity
 
-import saml2.attributemaps as attributemaps
-
-from saml2.mdstore import destinations
+from saml2.mdstore import locations
 from saml2.profile import paos, ecp
+from saml2.saml import NAMEID_FORMAT_PERSISTENT
 from saml2.saml import NAMEID_FORMAT_TRANSIENT
-from saml2.samlp import AuthnQuery, RequestedAuthnContext
+from saml2.saml import AuthnContextClassRef
+from saml2.samlp import AuthnQuery
+from saml2.samlp import RequestedAuthnContext
 from saml2.samlp import NameIDMappingRequest
 from saml2.samlp import AttributeQuery
 from saml2.samlp import AuthzDecisionQuery
 from saml2.samlp import AuthnRequest
 from saml2.samlp import Extensions
 from saml2.extension import sp_type
-from saml2.extension import requested_attributes
+from saml2.extension.requested_attributes import RequestedAttribute
+from saml2.extension.requested_attributes import RequestedAttributes
 
 import saml2
 from saml2.soap import make_soap_enveloped_saml_thingy
@@ -53,6 +56,9 @@ from saml2 import BINDING_HTTP_REDIRECT
 from saml2 import BINDING_HTTP_POST
 from saml2 import BINDING_PAOS
 
+from saml2.xmldsig import SIG_ALLOWED_ALG
+from saml2.xmldsig import DIGEST_ALLOWED_ALG
+
 
 logger = logging.getLogger(__name__)
 
@@ -91,6 +97,52 @@ class NoServiceDefined(SAMLError):
     pass
 
 
+def create_requested_attribute_node(requested_attrs, attribute_converters):
+    items = []
+    for attr in requested_attrs:
+        friendly_name = attr.get('friendly_name')
+        name = attr.get('name')
+        name_format = attr.get('name_format')
+        is_required = str(attr.get('required', False)).lower()
+
+        if not name and not friendly_name:
+            raise ValueError("Missing required attribute: 'name' or 'friendly_name'")
+
+        if not name:
+            for converter in attribute_converters:
+                try:
+                    name = converter._to[friendly_name.lower()]
+                except KeyError:
+                    continue
+                else:
+                    if not name_format:
+                        name_format = converter.name_format
+                    break
+
+        if not friendly_name:
+            for converter in attribute_converters:
+                try:
+                    friendly_name = converter._fro[name.lower()]
+                except KeyError:
+                    continue
+                else:
+                    if not name_format:
+                        name_format = converter.name_format
+                    break
+
+        items.append(
+            RequestedAttribute(
+                is_required=is_required,
+                name_format=name_format,
+                friendly_name=friendly_name,
+                name=name,
+            )
+        )
+
+    node = RequestedAttributes(extension_elements=items)
+    return node
+
+
 class Base(Entity):
     """ The basic pySAML2 service provider class """
 
@@ -116,23 +168,22 @@ class Base(Entity):
 
         attribute_defaults = {
             "logout_requests_signed": False,
+            "logout_responses_signed": False,
             "allow_unsolicited": False,
             "authn_requests_signed": False,
             "want_assertions_signed": False,
             "want_response_signed": True,
-            "want_assertions_or_response_signed" : False
+            "want_assertions_or_response_signed": False,
         }
-
         for attr, val_default in attribute_defaults.items():
             val_config = self.config.getattr(attr, "sp")
-            if val_config is None:
-                val = val_default
-            else:
-                val = val_config
-
+            val = (
+                val_config
+                if val_config is not None
+                else val_default
+            )
             if val == 'true':
                 val = True
-
             setattr(self, attr, val)
 
         if self.entity_type == "sp" and not any(
@@ -142,10 +193,15 @@ class Base(Entity):
                 self.want_assertions_or_response_signed,
             ]
         ):
-            logger.warning(
-                "The SAML service provider accepts unsigned SAML Responses "
-                "and Assertions. This configuration is insecure."
+            warn_msg = (
+                "The SAML service provider accepts "
+                "unsigned SAML Responses and Assertions. "
+                "This configuration is insecure. "
+                "Consider setting want_assertions_signed, want_response_signed "
+                "or want_assertions_or_response_signed configuration options."
             )
+            logger.warning(warn_msg)
+            _warn(warn_msg)
 
         self.artifact2response = {}
 
@@ -166,7 +222,7 @@ class Base(Entity):
             # verify that it's in the metadata
             srvs = self.metadata.single_sign_on_service(entityid, binding)
             if srvs:
-                return destinations(srvs)[0]
+                return next(locations(srvs), None)
             else:
                 logger.info("_sso_location: %s, %s", entityid, binding)
                 raise IdpUnspecified("No IdP to send to given the premises")
@@ -178,9 +234,9 @@ class Base(Entity):
             raise IdpUnspecified("Too many IdPs to choose from: %s" % eids)
 
         try:
-            srvs = self.metadata.single_sign_on_service(list(eids.keys())[0],
-                                                        binding)
-            return destinations(srvs)[0]
+            srvs = self.metadata.single_sign_on_service(list(eids.keys())[0], binding)
+            return next(locations(srvs), None)
+
         except IndexError:
             raise IdpUnspecified("No IdP to send to given the premises")
 
@@ -229,20 +285,33 @@ class Base(Entity):
         else:
             return None
 
-    def create_authn_request(self, destination, vorg="", scoping=None,
-            binding=saml2.BINDING_HTTP_POST,
-            nameid_format=None,
-            service_url_binding=None, message_id=0,
-            consent=None, extensions=None, sign=None,
-            allow_create=None, sign_prepare=False, sign_alg=None,
-            digest_alg=None, **kwargs):
+    # XXX DONE sp create > _message
+    def create_authn_request(
+        self,
+        destination,
+        vorg="",
+        scoping=None,
+        binding=BINDING_HTTP_POST,
+        nameid_format=None,
+        service_url_binding=None,
+        message_id=0,
+        consent=None,
+        extensions=None,
+        sign=None,
+        sign_prepare=None,
+        sign_alg=None,
+        digest_alg=None,
+        allow_create=None,
+        requested_attributes=None,
+        **kwargs,
+    ):
         """ Creates an authentication request.
 
         :param destination: Where the request should be sent.
         :param vorg: The virtual organization the service belongs to.
         :param scoping: The scope of the request
         :param binding: The protocol to use for the Response !!
-        :param nameid_format: Format of the NameID
+        :param nameid_format: Format of the NameIDPolicy
         :param service_url_binding: Where the reply should be sent dependent
             on reply binding.
         :param message_id: The identifier for this request
@@ -250,127 +319,115 @@ class Base(Entity):
         :param extensions: Possible extensions
         :param sign: Whether the request should be signed or not.
         :param sign_prepare: Whether the signature should be prepared or not.
+        :param sign_alg: The request signature algorithm
+        :param digest_alg: The request digest algorithm
         :param allow_create: If the identity provider is allowed, in the course
             of fulfilling the request, to create a new identifier to represent
             the principal.
+        :param requested_attributes: A list of dicts which define attributes to
+            be used as eIDAS Requested Attributes for this request. If not
+            defined the configuration option requested_attributes will be used,
+            if defined. The format is the same as the requested_attributes
+            configuration option.
         :param kwargs: Extra key word arguments
         :return: either a tuple of request ID and <samlp:AuthnRequest> instance
                  or a tuple of request ID and str when sign is set to True
         """
-        client_crt = None
-        if "client_crt" in kwargs:
-            client_crt = kwargs["client_crt"]
-
         args = {}
 
-        if self.config.getattr('hide_assertion_consumer_service', 'sp'):
+        # AssertionConsumerServiceURL
+        # AssertionConsumerServiceIndex
+        hide_assertion_consumer_service = self.config.getattr('hide_assertion_consumer_service', 'sp')
+        assertion_consumer_service_url = (
+            kwargs.pop("assertion_consumer_service_urls", [None])[0]
+            or kwargs.pop("assertion_consumer_service_url", None)
+        )
+        assertion_consumer_service_index = kwargs.pop("assertion_consumer_service_index", None)
+        service_url = (self.service_urls(service_url_binding or binding) or [None])[0]
+        if hide_assertion_consumer_service:
             args["assertion_consumer_service_url"] = None
             binding = None
-        else:
-            try:
-                args["assertion_consumer_service_url"] = kwargs[
-                    "assertion_consumer_service_urls"][0]
-                del kwargs["assertion_consumer_service_urls"]
-            except KeyError:
-                try:
-                    args["assertion_consumer_service_url"] = kwargs[
-                        "assertion_consumer_service_url"]
-                    del kwargs["assertion_consumer_service_url"]
-                except KeyError:
-                    try:
-                        args["assertion_consumer_service_index"] = str(
-                            kwargs["assertion_consumer_service_index"])
-                        del kwargs["assertion_consumer_service_index"]
-                    except KeyError:
-                        if service_url_binding is None:
-                            service_urls = self.service_urls(binding)
-                        else:
-                            service_urls = self.service_urls(service_url_binding)
-                        args["assertion_consumer_service_url"] = service_urls[0]
+        elif assertion_consumer_service_url:
+            args["assertion_consumer_service_url"] = assertion_consumer_service_url
+        elif assertion_consumer_service_index:
+            args["assertion_consumer_service_index"] = assertion_consumer_service_index
+        elif service_url:
+            args["assertion_consumer_service_url"] = service_url
 
-        try:
-            args["provider_name"] = kwargs["provider_name"]
-        except KeyError:
-            if binding == BINDING_PAOS:
-                pass
-            else:
-                args["provider_name"] = self._my_name()
+        # ProviderName
+        provider_name = kwargs.get("provider_name")
+        if not provider_name and binding != BINDING_PAOS:
+            provider_name = self._my_name()
+        args["provider_name"] = provider_name
+
+        requested_authn_context = (
+            kwargs.pop("requested_authn_context", None)
+            or self.config.getattr("requested_authn_context", "sp")
+            or {}
+        )
+        requested_authn_context_accrs = requested_authn_context.get(
+            "authn_context_class_ref", []
+        )
+        requested_authn_context_comparison = requested_authn_context.get(
+            "comparison", "exact"
+        )
+        if requested_authn_context_accrs:
+            args["requested_authn_context"] = RequestedAuthnContext(
+                authn_context_class_ref=[
+                    AuthnContextClassRef(accr)
+                    for accr in requested_authn_context_accrs
+                ],
+                comparison=requested_authn_context_comparison,
+            )
 
         # Allow argument values either as class instances or as dictionaries
         # all of these have cardinality 0..1
         _msg = AuthnRequest()
-        for param in ["scoping", "requested_authn_context", "conditions",
-                      "subject"]:
-            try:
-                _item = kwargs[param]
-            except KeyError:
-                pass
+        for param in ["scoping", "conditions", "subject"]:
+            _item = kwargs.pop(param, None)
+            if not _item:
+                continue
+
+            if isinstance(_item, _msg.child_class(param)):
+                args[param] = _item
             else:
-                del kwargs[param]
-                # either class instance or argument dictionary
-                if isinstance(_item, _msg.child_class(param)):
-                    args[param] = _item
-                elif isinstance(_item, dict):
-                    args[param] = RequestedAuthnContext(**_item)
-                else:
-                    raise ValueError("%s or wrong type expected %s" % (_item,
-                                                                       param))
+                raise ValueError("Wrong type for param {name}".format(name=param))
 
-        try:
-            args["name_id_policy"] = kwargs["name_id_policy"]
-            del kwargs["name_id_policy"]
-        except KeyError:
-            if allow_create is None:
-                allow_create = self.config.getattr("name_id_format_allow_create", "sp")
-                if allow_create is None:
-                    allow_create = "false"
-                else:
-                    if allow_create is True:
-                        allow_create = "true"
-                    else:
-                        allow_create = "false"
+        # NameIDPolicy
+        nameid_policy_format_config = self.config.getattr("name_id_policy_format", "sp")
+        nameid_policy_format = (
+            nameid_format
+            or nameid_policy_format_config
+            or None
+        )
 
-            if nameid_format == "":
-                name_id_policy = None
-            else:
-                if nameid_format is None:
-                    nameid_format = self.config.getattr("name_id_format", "sp")
+        allow_create_config = self.config.getattr("name_id_format_allow_create", "sp")
+        allow_create = (
+            None
+            # SAML 2.0 errata says AllowCreate MUST NOT be used for transient ids
+            if nameid_policy_format == NAMEID_FORMAT_TRANSIENT
+            else allow_create
+            if allow_create
+            else str(bool(allow_create_config)).lower()
+        )
 
-                    # If no nameid_format has been set in the configuration
-                    # or passed in then transient is the default.
-                    if nameid_format is None:
-                        # SAML 2.0 errata says AllowCreate MUST NOT be used for
-                        # transient ids - to make a conservative change this is
-                        # only applied for the default cause
-                        allow_create = None
-                        nameid_format = NAMEID_FORMAT_TRANSIENT
+        name_id_policy = (
+            kwargs.pop("name_id_policy", None)
+            if "name_id_policy" in kwargs
+            else None
+            if not nameid_policy_format
+            else samlp.NameIDPolicy(
+                allow_create=allow_create, format=nameid_policy_format
+            )
+        )
 
-                    # If a list has been configured or passed in choose the
-                    # first since NameIDPolicy can only have one format specified.
-                    elif isinstance(nameid_format, list):
-                        nameid_format = nameid_format[0]
+        if name_id_policy and vorg:
+            name_id_policy.sp_name_qualifier = vorg
+            name_id_policy.format = nameid_policy_format or NAMEID_FORMAT_PERSISTENT
 
-                    # Allow a deployer to signal that no format should be specified
-                    # in the NameIDPolicy by passing in or configuring the string 'None'.
-                    elif nameid_format == 'None':
-                        nameid_format = None
-
-                name_id_policy = samlp.NameIDPolicy(allow_create=allow_create,
-                                                    format=nameid_format)
-
-            if name_id_policy and vorg:
-                try:
-                    name_id_policy.sp_name_qualifier = vorg
-                    name_id_policy.format = saml.NAMEID_FORMAT_PERSISTENT
-                except KeyError:
-                    pass
-            args["name_id_policy"] = name_id_policy
-
-        try:
-            nsprefix = kwargs["nsprefix"]
-        except KeyError:
-            nsprefix = None
+        args["name_id_policy"] = name_id_policy
 
+        # eIDAS SPType
         conf_sp_type = self.config.getattr('sp_type', 'sp')
         conf_sp_type_in_md = self.config.getattr('sp_type_in_metadata', 'sp')
         if conf_sp_type and conf_sp_type_in_md is False:
@@ -379,59 +436,21 @@ class Base(Entity):
             item = sp_type.SPType(text=conf_sp_type)
             extensions.add_extension_element(item)
 
-        requested_attrs = self.config.getattr('requested_attributes', 'sp')
+        # eIDAS RequestedAttributes
+        requested_attrs = (
+            requested_attributes
+            or self.config.getattr('requested_attributes', 'sp')
+            or []
+        )
         if requested_attrs:
+            req_attrs_node = create_requested_attribute_node(
+                requested_attrs, self.config.attribute_converters
+            )
             if not extensions:
                 extensions = Extensions()
+            extensions.add_extension_element(req_attrs_node)
 
-            attributemapsmods = []
-            for modname in attributemaps.__all__:
-                attributemapsmods.append(getattr(attributemaps, modname))
-
-            items = []
-            for attr in requested_attrs:
-                friendly_name = attr.get('friendly_name')
-                name = attr.get('name')
-                name_format = attr.get('name_format')
-                is_required = str(attr.get('required', False)).lower()
-
-                if not name and not friendly_name:
-                    raise ValueError(
-                        "Missing required attribute: '{}' or '{}'".format(
-                            'name', 'friendly_name'))
-
-                if not name:
-                    for mod in attributemapsmods:
-                        try:
-                            name = mod.MAP['to'][friendly_name]
-                        except KeyError:
-                            continue
-                        else:
-                            if not name_format:
-                                name_format = mod.MAP['identifier']
-                            break
-
-                if not friendly_name:
-                    for mod in attributemapsmods:
-                        try:
-                            friendly_name = mod.MAP['fro'][name]
-                        except KeyError:
-                            continue
-                        else:
-                            if not name_format:
-                                name_format = mod.MAP['identifier']
-                            break
-
-                items.append(requested_attributes.RequestedAttribute(
-                    is_required=is_required,
-                    name_format=name_format,
-                    friendly_name=friendly_name,
-                    name=name))
-
-            item = requested_attributes.RequestedAttributes(
-                extension_elements=items)
-            extensions.add_extension_element(item)
-
+        # ForceAuthn
         force_authn = str(
             kwargs.pop("force_authn", None)
             or self.config.getattr("force_authn", "sp")
@@ -444,35 +463,44 @@ class Base(Entity):
                 AuthnRequest(), extensions, **kwargs
             )
             args.update(_args)
-
         args.pop("id", None)
 
-        if sign is None:
-            sign = self.authn_requests_signed
+        client_crt = kwargs.get("client_crt")
+        nsprefix = kwargs.get("nsprefix")
 
-        if (sign and self.sec.cert_handler.generate_cert()) or \
-                        client_crt is not None:
-            with self.lock:
-                self.sec.cert_handler.update_cert(True, client_crt)
-                if client_crt is not None:
-                    sign_prepare = True
-                return self._message(AuthnRequest, destination, message_id,
-                                     consent, extensions, sign, sign_prepare,
-                                     protocol_binding=binding,
-                                     scoping=scoping, nsprefix=nsprefix,
-                                     sign_alg=sign_alg, digest_alg=digest_alg,
-                                     **args)
-        return self._message(AuthnRequest, destination, message_id, consent,
-                             extensions, sign, sign_prepare,
-                             protocol_binding=binding,
-                             scoping=scoping, nsprefix=nsprefix,
-                             sign_alg=sign_alg, digest_alg=digest_alg, **args)
+        msg = self._message(
+            AuthnRequest,
+            destination,
+            message_id,
+            consent,
+            extensions,
+            sign,
+            sign_prepare,
+            sign_alg=sign_alg,
+            digest_alg=digest_alg,
+            protocol_binding=binding,
+            scoping=scoping,
+            nsprefix=nsprefix,
+            **args,
+        )
 
-    def create_attribute_query(self, destination, name_id=None,
-            attribute=None, message_id=0, consent=None,
-            extensions=None, sign=False, sign_prepare=False, sign_alg=None,
-            digest_alg=None,
-            **kwargs):
+        return msg
+
+    # XXX DONE sp create > _message
+    def create_attribute_query(
+        self,
+        destination,
+        name_id=None,
+        attribute=None,
+        message_id=0,
+        consent=None,
+        extensions=None,
+        sign=None,
+        sign_prepare=None,
+        sign_alg=None,
+        digest_alg=None,
+        **kwargs,
+    ):
         """ Constructs an AttributeQuery
 
         :param destination: To whom the query should be sent
@@ -526,18 +554,40 @@ class Base(Entity):
         except KeyError:
             nsprefix = None
 
-        return self._message(AttributeQuery, destination, message_id, consent,
-                             extensions, sign, sign_prepare, subject=subject,
-                             attribute=attribute, nsprefix=nsprefix,
-                             sign_alg=sign_alg, digest_alg=digest_alg)
+        return self._message(
+            AttributeQuery,
+            destination,
+            message_id,
+            consent,
+            extensions,
+            sign,
+            sign_prepare,
+            subject=subject,
+            attribute=attribute,
+            nsprefix=nsprefix,
+            sign_alg=sign_alg,
+            digest_alg=digest_alg,
+        )
 
     # MUST use SOAP for
     # AssertionIDRequest, SubjectQuery,
     # AuthnQuery, AttributeQuery, or AuthzDecisionQuery
-    def create_authz_decision_query(self, destination, action,
-            evidence=None, resource=None, subject=None,
-            message_id=0, consent=None, extensions=None,
-            sign=None, sign_alg=None, digest_alg=None, **kwargs):
+    # XXX DONE sp create > _message
+    def create_authz_decision_query(
+        self,
+        destination,
+        action,
+        evidence=None,
+        resource=None,
+        subject=None,
+        message_id=0,
+        consent=None,
+        extensions=None,
+        sign=None,
+        sign_alg=None,
+        digest_alg=None,
+        **kwargs,
+    ):
         """ Creates an authz decision query.
 
         :param destination: The IdP endpoint
@@ -552,19 +602,38 @@ class Base(Entity):
         :return: AuthzDecisionQuery instance
         """
 
-        return self._message(AuthzDecisionQuery, destination, message_id,
-                             consent, extensions, sign, action=action,
-                             evidence=evidence, resource=resource,
-                             subject=subject, sign_alg=sign_alg,
-                             digest_alg=digest_alg, **kwargs)
+        return self._message(
+            AuthzDecisionQuery,
+            destination,
+            message_id,
+            consent,
+            extensions,
+            sign,
+            action=action,
+            evidence=evidence,
+            resource=resource,
+            subject=subject,
+            sign_alg=sign_alg,
+            digest_alg=digest_alg,
+            **kwargs,
+        )
 
-    def create_authz_decision_query_using_assertion(self, destination,
-            assertion, action=None,
-            resource=None,
-            subject=None, message_id=0,
-            consent=None,
-            extensions=None,
-            sign=False, nsprefix=None):
+    # XXX DONE sp create > self.create_authz_decision_query (FIXME pass sign/sign_alg/etc) > _message
+    def create_authz_decision_query_using_assertion(
+        self,
+        destination,
+        assertion,
+        action=None,
+        resource=None,
+        subject=None,
+        message_id=0,
+        consent=None,
+        extensions=None,
+        sign=None,
+        sign_ag=None,
+        digest_alg=None,
+        nsprefix=None,
+    ):
         """ Makes an authz decision query based on a previously received
         Assertion.
 
@@ -589,9 +658,19 @@ class Base(Entity):
             _action = None
 
         return self.create_authz_decision_query(
-                destination, _action, saml.Evidence(assertion=assertion),
-                resource, subject, message_id=message_id, consent=consent,
-                extensions=extensions, sign=sign, nsprefix=nsprefix)
+            destination,
+            _action,
+            saml.Evidence(assertion=assertion),
+            resource,
+            subject,
+            message_id=message_id,
+            consent=consent,
+            extensions=extensions,
+            sign=sign,
+            sign_alg=sign_alg,
+            digest_alg=digest_alg,
+            nsprefix=nsprefix,
+        )
 
     @staticmethod
     def create_assertion_id_request(assertion_id_refs, **kwargs):
@@ -606,10 +685,21 @@ class Base(Entity):
         else:
             return 0, assertion_id_refs[0]
 
-    def create_authn_query(self, subject, destination=None, authn_context=None,
-            session_index="", message_id=0, consent=None,
-            extensions=None, sign=False, nsprefix=None, sign_alg=None,
-            digest_alg=None):
+    # XXX DONE sp create > _message
+    def create_authn_query(
+        self,
+        subject,
+        destination=None,
+        authn_context=None,
+        session_index="",
+        message_id=0,
+        consent=None,
+        extensions=None,
+        sign=None,
+        nsprefix=None,
+        sign_alg=None,
+        digest_alg=None,
+    ):
         """
 
         :param subject: The subject its all about as a <Subject> instance
@@ -622,19 +712,37 @@ class Base(Entity):
         :param sign: Whether the request should be signed or not.
         :return:
         """
-        return self._message(AuthnQuery, destination, message_id, consent,
-                             extensions, sign, subject=subject,
-                             session_index=session_index,
-                             requested_authn_context=authn_context,
-                             nsprefix=nsprefix, sign_alg=sign_alg,
-                             digest_alg=digest_alg)
+        return self._message(
+            AuthnQuery,
+            destination,
+            message_id,
+            consent,
+            extensions,
+            sign,
+            subject=subject,
+            session_index=session_index,
+            requested_authn_context=authn_context,
+            nsprefix=nsprefix,
+            sign_alg=sign_alg,
+            digest_alg=digest_alg,
+        )
 
-    def create_name_id_mapping_request(self, name_id_policy,
-            name_id=None, base_id=None,
-            encrypted_id=None, destination=None,
-            message_id=0, consent=None,
-            extensions=None, sign=False,
-            nsprefix=None, sign_alg=None, digest_alg=None):
+    # XXX DONE sp create > _message
+    def create_name_id_mapping_request(
+        self,
+        name_id_policy,
+        name_id=None,
+        base_id=None,
+        encrypted_id=None,
+        destination=None,
+        message_id=0,
+        consent=None,
+        extensions=None,
+        sign=None,
+        nsprefix=None,
+        sign_alg=None,
+        digest_alg=None,
+    ):
         """
 
         :param name_id_policy:
@@ -649,32 +757,44 @@ class Base(Entity):
         :return:
         """
 
-        # One of them must be present
-        assert name_id or base_id or encrypted_id
+        if not name_id and not base_id and not encrypted_id:
+            raise ValueError(
+                "At least one of name_id, base_id or encrypted_id must be present."
+            )
 
-        if name_id:
-            return self._message(NameIDMappingRequest, destination, message_id,
-                                 consent, extensions, sign,
-                                 name_id_policy=name_id_policy, name_id=name_id,
-                                 nsprefix=nsprefix, sign_alg=sign_alg,
-                                 digest_alg=digest_alg)
-        elif base_id:
-            return self._message(NameIDMappingRequest, destination, message_id,
-                                 consent, extensions, sign,
-                                 name_id_policy=name_id_policy, base_id=base_id,
-                                 nsprefix=nsprefix, sign_alg=sign_alg,
-                                 digest_alg=digest_alg)
-        else:
-            return self._message(NameIDMappingRequest, destination, message_id,
-                                 consent, extensions, sign,
-                                 name_id_policy=name_id_policy,
-                                 encrypted_id=encrypted_id, nsprefix=nsprefix,
-                                 sign_alg=sign_alg, digest_alg=digest_alg)
+        id_attr = {
+            "name_id": name_id,
+            "base_id": (
+                base_id
+                if not name_id
+                else None
+            ),
+            "encrypted_id": (
+                encrypted_id
+                if not name_id and not base_id
+                else None
+            ),
+        }
+
+        return self._message(
+            NameIDMappingRequest,
+            destination,
+            message_id,
+            consent,
+            extensions,
+            sign,
+            name_id_policy=name_id_policy,
+            **id_attr,
+            nsprefix=nsprefix,
+            sign_alg=sign_alg,
+            digest_alg=digest_alg,
+        )
 
     # ======== response handling ===========
 
-    def parse_authn_request_response(self, xmlstr, binding, outstanding=None,
-                                     outstanding_certs=None, conv_info=None):
+    def parse_authn_request_response(
+        self, xmlstr, binding, outstanding=None, outstanding_certs=None, conv_info=None
+    ):
         """ Deal with an AuthnResponse
 
         :param xmlstr: The reply as a xml string
@@ -703,15 +823,14 @@ class Base(Entity):
             "return_addrs": self.service_urls(binding=binding),
             "entity_id": self.config.entityid,
             "attribute_converters": self.config.attribute_converters,
-            "allow_unknown_attributes":
-                self.config.allow_unknown_attributes,
-            'conv_info': conv_info
+            "allow_unknown_attributes": self.config.allow_unknown_attributes,
+            'conv_info': conv_info,
         }
 
         try:
-            resp = self._parse_response(xmlstr, AuthnResponse,
-                                        "assertion_consumer_service",
-                                        binding, **kwargs)
+            resp = self._parse_response(
+                xmlstr, AuthnResponse, "assertion_consumer_service", binding, **kwargs
+            )
         except StatusError as err:
             logger.error("SAML status error: %s", err)
             raise
@@ -722,12 +841,14 @@ class Base(Entity):
             raise
 
         if not isinstance(resp, AuthnResponse):
-            logger.error("Response type not supported: %s",
-                         saml2.class_name(resp))
+            logger.error("Response type not supported: %s", saml2.class_name(resp))
             return None
 
-        if (resp.assertion and len(resp.response.encrypted_assertion) == 0 and
-                resp.assertion.subject.name_id):
+        if (
+            resp.assertion
+            and len(resp.response.encrypted_assertion) == 0
+            and resp.assertion.subject.name_id
+        ):
             self.users.add_information_about_person(resp.session_info())
             logger.info("--- ADDED person info ----")
 
@@ -737,15 +858,15 @@ class Base(Entity):
     # SubjectQuery, AuthnQuery, RequestedAuthnContext, AttributeQuery,
     # AuthzDecisionQuery all get Response as response
 
-    def parse_authz_decision_query_response(self, response,
-            binding=BINDING_SOAP):
+    def parse_authz_decision_query_response(self, response, binding=BINDING_SOAP):
         """ Verify that the response is OK
         """
-        kwargs = {"entity_id": self.config.entityid,
-                  "attribute_converters": self.config.attribute_converters}
+        kwargs = {
+            "entity_id": self.config.entityid,
+            "attribute_converters": self.config.attribute_converters,
+        }
 
-        return self._parse_response(response, AuthzResponse, "", binding,
-                                    **kwargs)
+        return self._parse_response(response, AuthzResponse, "", binding, **kwargs)
 
     def parse_authn_query_response(self, response, binding=BINDING_SOAP):
         """ Verify that the response is OK
@@ -788,8 +909,16 @@ class Base(Entity):
 
     # ------------------- ECP ------------------------------------------------
 
-    def create_ecp_authn_request(self, entityid=None, relay_state="",
-            sign=False, **kwargs):
+    # XXX DONE sp create > create_authn_request (FIXME DONE sign/sign_alg/etc) > _message
+    def create_ecp_authn_request(
+        self,
+        entityid=None,
+        relay_state="",
+        sign=None,
+        sign_alg=None,
+        digest_alg=None,
+        **kwargs,
+    ):
         """ Makes an authentication request.
 
         :param entityid: The entity ID of the IdP to send the request to
@@ -806,16 +935,22 @@ class Base(Entity):
 
         # must_understand and act according to the standard
         #
-        paos_request = paos.Request(must_understand="1", actor=ACTOR,
-                                    response_consumer_url=my_url,
-                                    service=ECP_SERVICE)
+        paos_request = paos.Request(
+            must_understand="1",
+            actor=ACTOR,
+            response_consumer_url=my_url,
+            service=ECP_SERVICE,
+        )
 
         # ----------------------------------------
         # <ecp:RelayState>
         # ----------------------------------------
 
-        relay_state = ecp.RelayState(actor=ACTOR, must_understand="1",
-                                     text=relay_state)
+        relay_state = ecp.RelayState(
+            actor=ACTOR,
+            must_understand="1",
+            text=relay_state,
+        )
 
         # ----------------------------------------
         # <samlp:AuthnRequest>
@@ -838,20 +973,27 @@ class Base(Entity):
 
             # The IDP publishes support for ECP by using the SOAP binding on
             # SingleSignOnService
-            _, location = self.pick_binding("single_sign_on_service",
-                                            [_binding], entity_id=entityid)
+            _, location = self.pick_binding(
+                "single_sign_on_service", [_binding], entity_id=entityid
+            )
             req_id, authn_req = self.create_authn_request(
-                    location, service_url_binding=BINDING_PAOS, **kwargs)
+                location,
+                service_url_binding=BINDING_PAOS,
+                sign=sign,
+                sign_alg=sign_alg,
+                digest_alg=digest_alg,
+                **kwargs,
+            )
 
         # ----------------------------------------
         # The SOAP envelope
         # ----------------------------------------
 
-        soap_envelope = make_soap_enveloped_saml_thingy(authn_req,
-                                                        [paos_request,
-                                                         relay_state])
+        soap_envelope = make_soap_enveloped_saml_thingy(
+            authn_req, [paos_request, relay_state]
+        )
 
-        return req_id, "%s" % soap_envelope
+        return req_id, str(soap_envelope)
 
     def parse_ecp_authn_response(self, txt, outstanding=None):
         rdict = soap.class_instances_from_soap_enveloped_saml_thingies(txt,
@@ -925,7 +1067,11 @@ class Base(Entity):
         }
 
         params = urlencode({k: v for k, v in args.items() if v})
-        return "%s?%s" % (url, params)
+        # url can already contain some parameters
+        if '?' in url:
+            return "%s&%s" % (url, params)
+        else:
+            return "%s?%s" % (url, params)
 
     @staticmethod
     def parse_discovery_service_response(url="", query="",
