@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-
 import calendar
 import logging
 import six
+
 from saml2.samlp import STATUS_VERSION_MISMATCH
 from saml2.samlp import STATUS_AUTHN_FAILED
 from saml2.samlp import STATUS_INVALID_ATTR_NAME_OR_VALUE
@@ -45,6 +45,7 @@ from saml2.saml import SCM_HOLDER_OF_KEY
 from saml2.saml import SCM_SENDER_VOUCHES
 from saml2.saml import encrypted_attribute_from_string
 from saml2.sigver import security_context
+from saml2.sigver import DecryptError
 from saml2.sigver import SignatureError
 from saml2.sigver import signed
 from saml2.attribute_converter import to_local
@@ -66,6 +67,10 @@ class IncorrectlySigned(SAMLError):
     pass
 
 
+class InvalidAssertion(SAMLError):
+    pass
+
+
 class DecryptionFailed(SAMLError):
     pass
 
@@ -93,6 +98,9 @@ class StatusAuthnFailed(StatusError):
 class StatusInvalidAttrNameOrValue(StatusError):
     pass
 
+class StatusInvalidAuthnResponseStatement(StatusError):
+    pass
+
 
 class StatusInvalidNameidPolicy(StatusError):
     pass
@@ -208,19 +216,17 @@ def for_me(conditions, myself):
         if not restriction.audience:
             continue
         for audience in restriction.audience:
-            if audience.text.strip() == myself:
+            if audience.text and audience.text.strip() == myself:
                 return True
             else:
-                # print("Not for me: %s != %s" % (audience.text.strip(),
-                # myself))
-                pass
-
+                logger.debug("AudienceRestriction - One condition not satisfied: {} != {}".format(audience.text, myself))
+    logger.debug("AudienceRestrictions not satisfied!")
     return False
 
 
 def authn_response(conf, return_addrs, outstanding_queries=None, timeslack=0,
-                   asynchop=True, allow_unsolicited=False,
-                   want_assertions_signed=False):
+        asynchop=True, allow_unsolicited=False,
+        want_assertions_signed=False, conv_info=None):
     sec = security_context(conf)
     if not timeslack:
         try:
@@ -231,12 +237,13 @@ def authn_response(conf, return_addrs, outstanding_queries=None, timeslack=0,
     return AuthnResponse(sec, conf.attribute_converters, conf.entityid,
                          return_addrs, outstanding_queries, timeslack,
                          asynchop=asynchop, allow_unsolicited=allow_unsolicited,
-                         want_assertions_signed=want_assertions_signed)
+                         want_assertions_signed=want_assertions_signed,
+                         conv_info=conv_info)
 
 
 # comes in over SOAP so synchronous
 def attribute_response(conf, return_addrs, timeslack=0, asynchop=False,
-                       test=False):
+        test=False, conv_info=None):
     sec = security_context(conf)
     if not timeslack:
         try:
@@ -246,16 +253,16 @@ def attribute_response(conf, return_addrs, timeslack=0, asynchop=False,
 
     return AttributeResponse(sec, conf.attribute_converters, conf.entityid,
                              return_addrs, timeslack, asynchop=asynchop,
-                             test=test)
+                             test=test, conv_info=conv_info)
 
 
 class StatusResponse(object):
     msgtype = "status_response"
 
     def __init__(self, sec_context, return_addrs=None, timeslack=0,
-                 request_id=0, asynchop=True):
+            request_id=0, asynchop=True, conv_info=None):
         self.sec = sec_context
-        self.return_addrs = return_addrs
+        self.return_addrs = return_addrs or []
 
         self.timeslack = timeslack
         self.request_id = request_id
@@ -269,9 +276,11 @@ class StatusResponse(object):
         self.signature_check = self.sec.correctly_signed_response
         self.require_signature = False
         self.require_response_signature = False
+        self.require_signature_or_response_signature = False
         self.not_signed = False
         self.asynchop = asynchop
         self.do_not_verify = False
+        self.conv_info = conv_info or {}
 
     def _clear(self):
         self.xmlstr = ""
@@ -316,7 +325,10 @@ class StatusResponse(object):
     def _loads(self, xmldata, decode=True, origxml=None):
 
         # own copy
-        self.xmlstr = xmldata[:]
+        if isinstance(xmldata, six.binary_type):
+            self.xmlstr = xmldata[:].decode('utf-8')
+        else:
+            self.xmlstr = xmldata[:]
         logger.debug("xmlstr: %s", self.xmlstr)
         if origxml:
             self.origxml = origxml
@@ -347,26 +359,29 @@ class StatusResponse(object):
         return self._postamble()
 
     def status_ok(self):
-        if self.response.status:
-            status = self.response.status
-            logger.info("status: %s", status)
-            if status.status_code.value != samlp.STATUS_SUCCESS:
-                logger.info("Not successful operation: %s", status)
-                if status.status_code.status_code:
-                    excep = STATUSCODE2EXCEPTION[
-                        status.status_code.status_code.value]
-                else:
-                    excep = StatusError
-                if status.status_message:
-                    msg = status.status_message.text
-                else:
-                    try:
-                        msg = status.status_code.status_code.value
-                    except Exception:
-                        msg = "Unknown error"
-                raise excep(
-                    "%s from %s" % (msg, status.status_code.value,))
-        return True
+        status = self.response.status
+        logger.info("status: %s", status)
+
+        if not status or status.status_code.value == samlp.STATUS_SUCCESS:
+            return True
+
+        err_code = (
+            status.status_code.status_code.value
+            if status.status_code.status_code
+            else None
+        )
+        err_msg = (
+            status.status_message.text
+            if status.status_message
+            else err_code or "Unknown error"
+        )
+        err_cls = STATUSCODE2EXCEPTION.get(err_code, StatusError)
+
+        msg = "Unsuccessful operation: {status}\n{msg} from {code}".format(
+            status=status, msg=err_msg, code=err_code
+        )
+        logger.info(msg)
+        raise err_cls(msg)
 
     def issue_instant_ok(self):
         """ Check that the response was issued at a reasonable time """
@@ -386,9 +401,7 @@ class StatusResponse(object):
                          self.in_response_to, self.request_id)
             return None
 
-        try:
-            assert self.response.version == "2.0"
-        except AssertionError:
+        if self.response.version != "2.0":
             _ver = float(self.response.version)
             if _ver < 2.0:
                 raise RequestVersionTooLow()
@@ -396,15 +409,15 @@ class StatusResponse(object):
                 raise RequestVersionTooHigh()
 
         if self.asynchop:
-            if self.response.destination and \
-                            self.response.destination not in self.return_addrs:
-                logger.error("%s not in %s", self.response.destination,
-                             self.return_addrs)
+            if (
+                self.response.destination
+                and self.response.destination not in self.return_addrs
+            ):
+                logger.error("destination '%s' not in return addresses '%s'", self.response.destination, self.return_addrs)
                 return None
 
-        assert self.issue_instant_ok()
-        assert self.status_ok()
-        return self
+        valid = self.issue_instant_ok() and self.status_ok()
+        return valid
 
     def loads(self, xmldata, decode=True, origxml=None):
         return self._loads(xmldata, decode, origxml)
@@ -422,16 +435,21 @@ class StatusResponse(object):
         self.response = mold.response
 
     def issuer(self):
-        return self.response.issuer.text.strip()
+        issuer_value = (
+            self.response.issuer.text
+            if self.response.issuer is not None
+            else ""
+        ).strip()
+        return issuer_value
 
 
 class LogoutResponse(StatusResponse):
     msgtype = "logout_response"
 
     def __init__(self, sec_context, return_addrs=None, timeslack=0,
-                 asynchop=True):
+            asynchop=True, conv_info=None):
         StatusResponse.__init__(self, sec_context, return_addrs, timeslack,
-                                asynchop=asynchop)
+                                asynchop=asynchop, conv_info=conv_info)
         self.signature_check = self.sec.correctly_signed_logout_response
 
 
@@ -439,9 +457,9 @@ class NameIDMappingResponse(StatusResponse):
     msgtype = "name_id_mapping_response"
 
     def __init__(self, sec_context, return_addrs=None, timeslack=0,
-                 request_id=0, asynchop=True):
+            request_id=0, asynchop=True, conv_info=None):
         StatusResponse.__init__(self, sec_context, return_addrs, timeslack,
-                                request_id, asynchop)
+                                request_id, asynchop, conv_info=conv_info)
         self.signature_check = self.sec \
             .correctly_signed_name_id_mapping_response
 
@@ -450,29 +468,30 @@ class ManageNameIDResponse(StatusResponse):
     msgtype = "manage_name_id_response"
 
     def __init__(self, sec_context, return_addrs=None, timeslack=0,
-                 request_id=0, asynchop=True):
+            request_id=0, asynchop=True, conv_info=None):
         StatusResponse.__init__(self, sec_context, return_addrs, timeslack,
-                                request_id, asynchop)
+                                request_id, asynchop, conv_info=conv_info)
         self.signature_check = self.sec.correctly_signed_manage_name_id_response
 
 
 # ----------------------------------------------------------------------------
 
-
 class AuthnResponse(StatusResponse):
     """ This is where all the profile compliance is checked.
     This one does saml2int compliance. """
     msgtype = "authn_response"
 
     def __init__(self, sec_context, attribute_converters, entity_id,
-                 return_addrs=None, outstanding_queries=None,
-                 timeslack=0, asynchop=True, allow_unsolicited=False,
-                 test=False, allow_unknown_attributes=False,
-                 want_assertions_signed=False, want_response_signed=False,
-                 **kwargs):
+            return_addrs=None, outstanding_queries=None,
+            timeslack=0, asynchop=True, allow_unsolicited=False,
+            test=False, allow_unknown_attributes=False,
+            want_assertions_signed=False,
+            want_assertions_or_response_signed=False,
+            want_response_signed=False,
+            conv_info=None, **kwargs):
 
         StatusResponse.__init__(self, sec_context, return_addrs, timeslack,
-                                asynchop=asynchop)
+                                asynchop=asynchop, conv_info=conv_info)
         self.entity_id = entity_id
         self.attribute_converters = attribute_converters
         if outstanding_queries:
@@ -487,6 +506,7 @@ class AuthnResponse(StatusResponse):
         self.session_not_on_or_after = 0
         self.allow_unsolicited = allow_unsolicited
         self.require_signature = want_assertions_signed
+        self.require_signature_or_response_signature = want_assertions_or_response_signed
         self.require_response_signature = want_response_signed
         self.test = test
         self.allow_unknown_attributes = allow_unknown_attributes
@@ -499,9 +519,7 @@ class AuthnResponse(StatusResponse):
     def check_subject_confirmation_in_response_to(self, irp):
         for assertion in self.response.assertion:
             for _sc in assertion.subject.subject_confirmation:
-                try:
-                    assert _sc.subject_confirmation_data.in_response_to == irp
-                except AssertionError:
+                if _sc.subject_confirmation_data.in_response_to != irp:
                     return False
 
         return True
@@ -516,17 +534,14 @@ class AuthnResponse(StatusResponse):
                 try:
                     if not self.check_subject_confirmation_in_response_to(
                             self.in_response_to):
-                        logger.exception(
-                            "Unsolicited response %s" % self.in_response_to)
                         raise UnsolicitedResponse(
                             "Unsolicited response: %s" % self.in_response_to)
                 except AttributeError:
                     pass
             elif self.allow_unsolicited:
+                # Should check that I haven't seen this before
                 pass
             else:
-                logger.exception(
-                    "Unsolicited response %s" % self.in_response_to)
                 raise UnsolicitedResponse(
                     "Unsolicited response: %s" % self.in_response_to)
 
@@ -539,15 +554,13 @@ class AuthnResponse(StatusResponse):
         self.assertion = None
 
     def authn_statement_ok(self, optional=False):
-        try:
-            # the assertion MUST contain one AuthNStatement
-            assert len(self.assertion.authn_statement) == 1
-        except AssertionError:
+        n_authn_statements = len(self.assertion.authn_statement)
+        if n_authn_statements != 1:
             if optional:
                 return True
             else:
-                logger.error("No AuthnStatement")
-                raise
+                msg = "Invalid number of AuthnStatement found in Response: {n}".format(n=n_authn_statements)
+                raise ValueError(msg)
 
         authn_statement = self.assertion.authn_statement[0]
         if authn_statement.session_not_on_or_after:
@@ -562,11 +575,14 @@ class AuthnResponse(StatusResponse):
         # check authn_statement.session_index
 
     def condition_ok(self, lax=False):
+        if not self.assertion.conditions:
+            # Conditions is Optional for Assertion, so, if it's absent, then we
+            # assume that its valid
+            return True
+
         if self.test:
             lax = True
 
-        # The Identity Provider MUST include a <saml:Conditions> element
-        assert self.assertion.conditions
         conditions = self.assertion.conditions
 
         logger.debug("conditions: %s", conditions)
@@ -595,10 +611,9 @@ class AuthnResponse(StatusResponse):
             else:
                 self.not_on_or_after = 0
 
-        if not self.allow_unsolicited:
-            if not for_me(conditions, self.entity_id):
-                if not lax:
-                    raise Exception("Not for me!!!")
+        if not for_me(conditions, self.entity_id):
+            if not lax:
+                raise Exception("AudienceRestrictions conditions not satisfied! (Local entity_id=%s)" % self.entity_id)
 
         if conditions.condition:  # extra conditions
             for cond in conditions.condition:
@@ -613,7 +628,7 @@ class AuthnResponse(StatusResponse):
 
         return True
 
-    def decrypt_attributes(self, attribute_statement):
+    def decrypt_attributes(self, attribute_statement, keys=None):
         """
         Decrypts possible encrypted attributes and adds the decrypts to the
         list of attributes.
@@ -627,11 +642,11 @@ class AuthnResponse(StatusResponse):
 
         for encattr in attribute_statement.encrypted_attribute:
             if not encattr.encrypted_key:
-                _decr = self.sec.decrypt(encattr.encrypted_data)
+                _decr = self.sec.decrypt_keys(encattr.encrypted_data, keys=keys)
                 _attr = attribute_from_string(_decr)
                 attribute_statement.attribute.append(_attr)
             else:
-                _decr = self.sec.decrypt(encattr)
+                _decr = self.sec.decrypt_keys(encattr, keys=keys)
                 enc_attr = encrypted_attribute_from_string(_decr)
                 attrlist = enc_attr.extensions_as_elements("Attribute", saml)
                 attribute_statement.attribute.extend(attrlist)
@@ -646,7 +661,7 @@ class AuthnResponse(StatusResponse):
                         self.allow_unknown_attributes)
 
     def get_identity(self):
-        """ The assertion can contain zero or one attributeStatements
+        """ The assertion can contain zero or more attributeStatements
 
         """
         ava = {}
@@ -655,15 +670,19 @@ class AuthnResponse(StatusResponse):
                 if _assertion.advice.assertion:
                     for tmp_assertion in _assertion.advice.assertion:
                         if tmp_assertion.attribute_statement:
-                            assert len(tmp_assertion.attribute_statement) == 1
-                            ava.update(self.read_attribute_statement(
-                                tmp_assertion.attribute_statement[0]))
+                            n_attr_statements = len(tmp_assertion.attribute_statement)
+                            if n_attr_statements != 1:
+                                msg = "Invalid number of AuthnStatement found in Response: {n}".format(n=n_attr_statements)
+                                raise ValueError(msg)
+                            ava.update(self.read_attribute_statement(tmp_assertion.attribute_statement[0]))
             if _assertion.attribute_statement:
-                assert len(_assertion.attribute_statement) == 1
-                _attr_statem = _assertion.attribute_statement[0]
-                ava.update(self.read_attribute_statement(_attr_statem))
+                logger.debug("Assertion contains %s attribute statement(s)",
+                             (len(self.assertion.attribute_statement)))
+                for _attr_statem in _assertion.attribute_statement:
+                    logger.debug("Attribute Statement: %s" % (_attr_statem,))
+                    ava.update(self.read_attribute_statement(_attr_statem))
             if not ava:
-                logger.error("Missing Attribute Statement")
+                logger.debug("Assertion contains no attribute statements")
         return ava
 
     def _bearer_confirmed(self, data):
@@ -704,23 +723,36 @@ class AuthnResponse(StatusResponse):
         return True
 
     def _holder_of_key_confirmed(self, data):
-        if not data:
+        if not data or not data.extension_elements:
             return False
 
         has_keyinfo = False
-        for element in extension_elements_to_elements(data,
+        for element in extension_elements_to_elements(data.extension_elements,
                                                       [samlp, saml, xenc, ds]):
             if isinstance(element, ds.KeyInfo):
                 has_keyinfo = True
 
         return has_keyinfo
 
-    def get_subject(self):
+    def get_subject(self, keys=None):
         """ The assertion must contain a Subject
         """
-        assert self.assertion.subject
+
+        if not self.assertion:
+            raise ValueError("Missing assertion")
+
+        if not self.assertion.subject:
+            raise ValueError(
+                "Invalid assertion subject: {subject}".format(
+                    subject=self.assertion.subject
+                )
+            )
         subject = self.assertion.subject
         subjconf = []
+
+        if not self.verify_attesting_entity(subject.subject_confirmation):
+            raise VerificationError("No valid attesting address")
+
         for subject_confirmation in subject.subject_confirmation:
             _data = subject_confirmation.subject_confirmation_data
 
@@ -736,6 +768,10 @@ class AuthnResponse(StatusResponse):
                 raise ValueError("Unknown subject confirmation method: %s" % (
                     subject_confirmation.method,))
 
+            _recip = _data.recipient
+            if not _recip or not self.verify_recipient(_recip):
+                raise VerificationError("No valid recipient")
+
             subjconf.append(subject_confirmation)
 
         if not subjconf:
@@ -749,8 +785,9 @@ class AuthnResponse(StatusResponse):
             self.name_id = subject.name_id
         elif subject.encrypted_id:
             # decrypt encrypted ID
-            _name_id_str = self.sec.decrypt(
-                subject.encrypted_id.encrypted_data.to_string())
+            _name_id_str = self.sec.decrypt_keys(
+                subject.encrypted_id.encrypted_data.to_string(), keys=keys
+            )
             _name_id = saml.name_id_from_string(_name_id_str)
             self.name_id = _name_id
 
@@ -811,7 +848,7 @@ class AuthnResponse(StatusResponse):
             raise
 
     def decrypt_assertions(self, encrypted_assertions, decr_txt, issuer=None,
-                           verified=False):
+            verified=False):
         """ Moves the decrypted assertion from the encrypted assertion to a
         list.
 
@@ -871,7 +908,6 @@ class AuthnResponse(StatusResponse):
         :param resp: A saml response.
         :return: True encrypted data exists otherwise false.
         """
-        _has_encrypt_data = False
         if resp.encrypted_assertion:
             res = self.find_encrypt_data_assertion(resp.encrypted_assertion)
             if res:
@@ -896,21 +932,16 @@ class AuthnResponse(StatusResponse):
         if self.context == "AuthnQuery":
             # can contain one or more assertions
             pass
-        else:  # This is a saml2int limitation
-            try:
-                assert len(self.response.assertion) == 1 or \
-                       len(self.response.encrypted_assertion) == 1
-            except AssertionError:
-                raise Exception("No assertion part")
-
-        has_encrypted_assertions = self.find_encrypt_data(self.response)  #
-        # self.response.encrypted_assertion
-        # if not has_encrypted_assertions and self.response.assertion:
-        #    for tmp_assertion in self.response.assertion:
-        #        if tmp_assertion.advice:
-        #            if tmp_assertion.advice.encrypted_assertion:
-        #                has_encrypted_assertions = True
-        #                break
+        else:
+            # This is a saml2int limitation
+            n_assertions = len(self.response.assertion)
+            n_assertions_enc = len(self.response.encrypted_assertion)
+            if n_assertions != 1 and n_assertions_enc != 1 and self.assertion is None:
+                raise InvalidAssertion(
+                    "Invalid number of assertions in Response: {n}".format(
+                        n=n_assertions+n_assertions_enc
+                    )
+                )
 
         if self.response.assertion:
             logger.debug("***Unencrypted assertion***")
@@ -918,33 +949,58 @@ class AuthnResponse(StatusResponse):
                 if not self._assertion(assertion, False):
                     return False
 
-        if has_encrypted_assertions:
-            _enc_assertions = []
+        if self.find_encrypt_data(self.response):
             logger.debug("***Encrypted assertion/-s***")
-            decr_text = "%s" % self.response
+            _enc_assertions = []
             resp = self.response
+            decr_text = str(self.response)
+
             decr_text_old = None
             while self.find_encrypt_data(resp) and decr_text_old != decr_text:
                 decr_text_old = decr_text
-                decr_text = self.sec.decrypt_keys(decr_text, keys)
-                resp = samlp.response_from_string(decr_text)
-            _enc_assertions = self.decrypt_assertions(resp.encrypted_assertion,
-                                                      decr_text)
+                try:
+                    decr_text = self.sec.decrypt_keys(decr_text, keys=keys)
+                except DecryptError as e:
+                    continue
+                else:
+                    resp = samlp.response_from_string(decr_text)
+                    # check and prepare for comparison between str and unicode
+                    if type(decr_text_old) != type(decr_text):
+                        if isinstance(decr_text_old, six.binary_type):
+                            decr_text_old = decr_text_old.decode("utf-8")
+                        else:
+                            decr_text_old = decr_text_old.encode("utf-8")
+
+            _enc_assertions = self.decrypt_assertions(
+                resp.encrypted_assertion, decr_text
+            )
+
             decr_text_old = None
-            while (self.find_encrypt_data(
-                    resp) or self.find_encrypt_data_assertion_list(
-                    _enc_assertions)) and \
-                            decr_text_old != decr_text:
+            while (
+                self.find_encrypt_data(resp)
+                or self.find_encrypt_data_assertion_list(_enc_assertions)
+            ) and decr_text_old != decr_text:
                 decr_text_old = decr_text
-                decr_text = self.sec.decrypt_keys(decr_text, keys)
-                resp = samlp.response_from_string(decr_text)
-                _enc_assertions = self.decrypt_assertions(
-                    resp.encrypted_assertion, decr_text, verified=True)
-            # _enc_assertions = self.decrypt_assertions(
-            # resp.encrypted_assertion, decr_text, verified=True)
+                try:
+                    decr_text = self.sec.decrypt_keys(decr_text, keys=keys)
+                except DecryptError as e:
+                    continue
+                else:
+                    resp = samlp.response_from_string(decr_text)
+                    _enc_assertions = self.decrypt_assertions(
+                        resp.encrypted_assertion, decr_text, verified=True
+                    )
+                    # check and prepare for comparison between str and unicode
+                    if type(decr_text_old) != type(decr_text):
+                        if isinstance(decr_text_old, six.binary_type):
+                            decr_text_old = decr_text_old.decode("utf-8")
+                        else:
+                            decr_text_old = decr_text_old.encode("utf-8")
+
             all_assertions = _enc_assertions
             if resp.assertion:
                 all_assertions = all_assertions + resp.assertion
+
             if len(all_assertions) > 0:
                 for tmp_ass in all_assertions:
                     if tmp_ass.advice and tmp_ass.advice.encrypted_assertion:
@@ -959,6 +1015,7 @@ class AuthnResponse(StatusResponse):
                             tmp_ass.advice.assertion = advice_res
                         if len(advice_res) > 0:
                             tmp_ass.advice.encrypted_assertion = []
+
             self.response.assertion = resp.assertion
             for assertion in _enc_assertions:
                 if not self._assertion(assertion, True):
@@ -984,9 +1041,11 @@ class AuthnResponse(StatusResponse):
         return True
 
     def verify(self, keys=None):
-        """ Verify that the assertion is syntactically correct and
-        the signature is correct if present.
-        :param key_file: If not the default key file should be used this is it.
+        """ Verify that the assertion is syntactically correct and the
+        signature is correct if present.
+
+        :param keys: If not the default key file should be used then use one
+        of these.
         """
 
         try:
@@ -995,7 +1054,7 @@ class AuthnResponse(StatusResponse):
             logger.error("Verification error on the response: %s", err)
             raise
         else:
-            if res is None:
+            if not res:
                 return None
 
         if not isinstance(self.response, samlp.Response):
@@ -1017,23 +1076,27 @@ class AuthnResponse(StatusResponse):
 
     def authn_info(self):
         res = []
-        for astat in self.assertion.authn_statement:
-            context = astat.authn_context
+        for statement in getattr(self.assertion, 'authn_statement', []):
+            authn_instant = getattr(statement, "authn_instant", "")
+
+            context = statement.authn_context
+            if not context:
+                continue
+
             try:
-                authn_instant = astat.authn_instant
+                authn_class = (
+                    context.authn_context_class_ref.text
+                    or context.authn_context_decl_ref.text
+                )
             except AttributeError:
-                authn_instant = ""
-            if context:
-                try:
-                    aclass = context.authn_context_class_ref.text
-                except AttributeError:
-                    aclass = ""
-                try:
-                    authn_auth = [a.text for a in
-                                  context.authenticating_authority]
-                except AttributeError:
-                    authn_auth = []
-                res.append((aclass, authn_auth, authn_instant))
+                authn_class = ""
+
+            authenticating_authorities = getattr(
+                context, "authenticating_authority", []
+            )
+            authn_auth = [authority.text for authority in authenticating_authorities]
+
+            res.append((authn_class, authn_auth, authn_instant))
         return res
 
     def authz_decision_info(self):
@@ -1057,33 +1120,68 @@ class AuthnResponse(StatusResponse):
             return {"name_id": self.name_id, "came_from": self.came_from,
                     "issuer": self.issuer(), "not_on_or_after": nooa,
                     "authz_decision_info": self.authz_decision_info()}
-        else:
+        elif getattr(self.assertion, 'authn_statement', None):
             authn_statement = self.assertion.authn_statement[0]
             return {"ava": self.ava, "name_id": self.name_id,
                     "came_from": self.came_from, "issuer": self.issuer(),
                     "not_on_or_after": nooa, "authn_info": self.authn_info(),
                     "session_index": authn_statement.session_index}
+        else:
+            raise StatusInvalidAuthnResponseStatement(
+                "The Authn Response Statement is not valid"
+            )
 
     def __str__(self):
-        if not isinstance(self.xmlstr, six.string_types):
-            return "%s" % self.xmlstr.decode("utf-8")
-        return "%s" % self.xmlstr
+        return self.xmlstr
 
-    def verify_attesting_entity(self, address):
+    def verify_recipient(self, recipient):
         """
-        Assumes one assertion. At least one address specification has to be
-        correct.
+        Verify that I'm the recipient of the assertion
 
-        :param address: IP address of attesting entity
+        :param recipient: A URI specifying the entity or location to which an
+            attesting entity can present the assertion.
+        :return: True/False
+        """
+        if not self.conv_info:
+            return True
+
+        _info = self.conv_info
+
+        try:
+            if recipient == _info['entity_id']:
+                return True
+        except KeyError:
+            pass
+
+        try:
+            if recipient in self.return_addrs:
+                return True
+        except KeyError:
+            pass
+
+        return False
+
+    def verify_attesting_entity(self, subject_confirmation):
+        """
+        At least one address specification has to be correct.
+
+        :param subject_confirmation: A SubbjectConfirmation instance
         :return: True/False
         """
 
+        try:
+            address = self.conv_info['remote_addr']
+        except KeyError:
+            address = '0.0.0.0'
+
         correct = 0
-        for subject_conf in self.assertion.subject.subject_confirmation:
+        for subject_conf in subject_confirmation:
             if subject_conf.subject_confirmation_data is None:
                 correct += 1  # In reality undefined
             elif subject_conf.subject_confirmation_data.address:
-                if subject_conf.subject_confirmation_data.address == address:
+                if address == '0.0.0.0':  # accept anything
+                    correct += 1
+                elif subject_conf.subject_confirmation_data.address == address:
                     correct += 1
             else:
                 correct += 1
@@ -1098,10 +1196,12 @@ class AuthnQueryResponse(AuthnResponse):
     msgtype = "authn_query_response"
 
     def __init__(self, sec_context, attribute_converters, entity_id,
-                 return_addrs=None, timeslack=0, asynchop=False, test=False):
+            return_addrs=None, timeslack=0, asynchop=False, test=False,
+            conv_info=None):
         AuthnResponse.__init__(self, sec_context, attribute_converters,
                                entity_id, return_addrs, timeslack=timeslack,
-                               asynchop=asynchop, test=test)
+                               asynchop=asynchop, test=test,
+                               conv_info=conv_info)
         self.entity_id = entity_id
         self.attribute_converters = attribute_converters
         self.assertion = None
@@ -1115,10 +1215,12 @@ class AttributeResponse(AuthnResponse):
     msgtype = "attribute_response"
 
     def __init__(self, sec_context, attribute_converters, entity_id,
-                 return_addrs=None, timeslack=0, asynchop=False, test=False):
+            return_addrs=None, timeslack=0, asynchop=False, test=False,
+            conv_info=None):
         AuthnResponse.__init__(self, sec_context, attribute_converters,
                                entity_id, return_addrs, timeslack=timeslack,
-                               asynchop=asynchop, test=test)
+                               asynchop=asynchop, test=test,
+                               conv_info=conv_info)
         self.entity_id = entity_id
         self.attribute_converters = attribute_converters
         self.assertion = None
@@ -1131,10 +1233,11 @@ class AuthzResponse(AuthnResponse):
     msgtype = "authz_decision_response"
 
     def __init__(self, sec_context, attribute_converters, entity_id,
-                 return_addrs=None, timeslack=0, asynchop=False):
+            return_addrs=None, timeslack=0, asynchop=False,
+            conv_info=None):
         AuthnResponse.__init__(self, sec_context, attribute_converters,
                                entity_id, return_addrs, timeslack=timeslack,
-                               asynchop=asynchop)
+                               asynchop=asynchop, conv_info=conv_info)
         self.entity_id = entity_id
         self.attribute_converters = attribute_converters
         self.assertion = None
@@ -1145,10 +1248,12 @@ class ArtifactResponse(AuthnResponse):
     msgtype = "artifact_response"
 
     def __init__(self, sec_context, attribute_converters, entity_id,
-                 return_addrs=None, timeslack=0, asynchop=False, test=False):
+            return_addrs=None, timeslack=0, asynchop=False, test=False,
+            conv_info=None):
         AuthnResponse.__init__(self, sec_context, attribute_converters,
                                entity_id, return_addrs, timeslack=timeslack,
-                               asynchop=asynchop, test=test)
+                               asynchop=asynchop, test=test,
+                               conv_info=conv_info)
         self.entity_id = entity_id
         self.attribute_converters = attribute_converters
         self.assertion = None
@@ -1156,9 +1261,9 @@ class ArtifactResponse(AuthnResponse):
 
 
 def response_factory(xmlstr, conf, return_addrs=None, outstanding_queries=None,
-                     timeslack=0, decode=True, request_id=0, origxml=None,
-                     asynchop=True, allow_unsolicited=False,
-                     want_assertions_signed=False):
+        timeslack=0, decode=True, request_id=0, origxml=None,
+        asynchop=True, allow_unsolicited=False,
+        want_assertions_signed=False, conv_info=None):
     sec_context = security_context(conf)
     if not timeslack:
         try:
@@ -1171,23 +1276,23 @@ def response_factory(xmlstr, conf, return_addrs=None, outstanding_queries=None,
     extension_schema = conf.extension_schema
 
     response = StatusResponse(sec_context, return_addrs, timeslack, request_id,
-                              asynchop)
+                              asynchop, conv_info=conv_info)
     try:
         response.loads(xmlstr, decode, origxml)
         if response.response.assertion or response.response.encrypted_assertion:
-            authnresp = AuthnResponse(sec_context, attribute_converters,
-                                      entity_id, return_addrs,
-                                      outstanding_queries, timeslack, asynchop,
-                                      allow_unsolicited,
-                                      extension_schema=extension_schema,
-                                      want_assertions_signed=want_assertions_signed)
+            authnresp = AuthnResponse(
+                sec_context, attribute_converters, entity_id, return_addrs,
+                outstanding_queries, timeslack, asynchop, allow_unsolicited,
+                extension_schema=extension_schema,
+                want_assertions_signed=want_assertions_signed,
+                conv_info=conv_info)
             authnresp.update(response)
             return authnresp
     except TypeError:
         response.signature_check = sec_context.correctly_signed_logout_response
         response.loads(xmlstr, decode, origxml)
         logoutresp = LogoutResponse(sec_context, return_addrs, timeslack,
-                                    asynchop=asynchop)
+                                    asynchop=asynchop, conv_info=conv_info)
         logoutresp.update(response)
         return logoutresp
 
@@ -1202,7 +1307,7 @@ class AssertionIDResponse(object):
     msgtype = "assertion_id_response"
 
     def __init__(self, sec_context, attribute_converters, timeslack=0,
-                 **kwargs):
+            **kwargs):
 
         self.sec = sec_context
         self.timeslack = timeslack
@@ -1216,6 +1321,14 @@ class AssertionIDResponse(object):
         self.context = "AssertionIdResponse"
         self.signature_check = self.sec.correctly_signed_assertion_id_response
 
+        # Because this class is not a subclass of StatusResponse we need
+        # to add these attributes directly so that the _parse_response()
+        # method of the Entity class can treat instances of this class
+        # like all other responses.
+        self.require_signature = False
+        self.require_response_signature = False
+        self.require_signature_or_response_signature = False
+
     def loads(self, xmldata, decode=True, origxml=None):
         # own copy
         self.xmlstr = xmldata[:]
